@@ -865,6 +865,37 @@ class XlPath(type(pathlib.Path())):
             dir = tempfile.gettempdir()
         return cls(tempfile.mktemp(suffix=suffix, dir=dir))
 
+    @classmethod
+    def init(cls, path, root=None, *, suffix=None):
+        """ 仿照原来File的初始化接口形式 """
+        p = XlPath(path)
+        if root:
+            p = XlPath(root) / p
+
+        if suffix:
+            if suffix[0] == '.':  # 使用.时强制修改后缀
+                p = p.with_suffix(suffix)
+            elif not p.suffix:
+                p = p.with_suffix('.' + suffix)
+
+        return p
+
+    @classmethod
+    def safe_init(cls, arg_in):
+        """ 输入任意类型的_in，会用比较安全的机制，判断其是否为一个有效的路径格式并初始化
+        初始化失败则返回None
+        """
+        try:
+            p = XlPath(arg_in)
+            p.is_file()  # 有些问题上一步不一定测的出来，要再补一个测试。具体存不存在是不是文件并不重要，而是使用这个能检查出问题。
+            return p
+        except (ValueError, TypeError, OSError, PermissionError):
+            # ValueError：文件名过长，代表输入很可能是一段文本，根本不是路径
+            # TypeError：不是str等正常的参数
+            # OSError：非法路径名，例如有 *? 等
+            # PermissionError: linux上访问无权限、不存在的路径
+            return None
+
     def start(self, *args, **kwargs):
         """ 使用关联的程序打开p，类似于双击的效果
 
@@ -1036,14 +1067,16 @@ class XlPath(type(pathlib.Path())):
         else:  # 非文件对象
             raise FileNotFoundError(f'{self} 文件不存在，无法读取。')
 
-    def write_auto(self, data, *args, **kwargs):
+    def write_auto(self, data, *args, if_exists=None, **kwargs):
         """ 根据文件后缀自动识别写入函数 """
         mode = self.suffix.lower()[1:]
         write_func = getattr(self, 'write_' + mode, None)
-        if write_func:
-            return write_func(data, *args, **kwargs)
-        else:
-            return self.write_text(str(data), *args, **kwargs)
+        if self.exist_preprcs(if_exists):
+            if write_func:
+                return write_func(data, *args, **kwargs)
+            else:
+                return self.write_text(str(data), *args, **kwargs)
+        return self
 
     def __2_glob(self):
         """ 类型判断、glob系列 """
@@ -1140,7 +1173,55 @@ class XlPath(type(pathlib.Path())):
         """ 复制、删除、移动等操作 """
         pass
 
-    def rename2(self, dst, *, if_exists=None):
+    def exist_preprcs(self, if_exists=None):
+        """ 从旧版File机制复制过来的函数
+
+        这个实际上是在做copy等操作前，如果目标文件已存在，需要预先删除等的预处理
+        并返回判断，是否需要执行下一步操作
+
+        有时候情况比较复杂，process无法满足需求时，可以用exist_preprcs这个底层函数协助
+
+        :param if_exists:
+            None: 不做任何处理，直接运行，依赖于功能本身是否有覆盖写入机制
+            'error': 如果要替换的目标文件已经存在，则报错
+            'replace': 把存在的文件先删除
+                本来是叫'delete'更准确的，但是考虑用户理解，
+                    一般都是用在文件替换场合，叫成'delete'会非常怪异，带来不必要的困扰、误解
+                所以还是决定叫'replace'
+            'skip': 不执行后续功能
+            'backup': 先做备份  （对原文件先做一个备份）
+        """
+        need_run = True
+        if self.exists():
+            if if_exists is None:
+                return need_run
+            elif if_exists == 'error':
+                raise FileExistsError(f'目标文件已存在： {self}')
+            elif if_exists == 'replace':
+                self.delete()
+            elif if_exists == 'skip':
+                need_run = False
+            elif if_exists == 'backup':
+                self.backup(move=True)
+            else:
+                raise ValueError(f'{if_exists}')
+        return need_run
+
+    def copy(self, dst, if_exists=None):
+        if not self.exists():
+            return
+
+        dst = XlPath(dst)
+        if dst.exist_preprcs(if_exists):
+            if self.is_file():
+                shutil.copy2(self, dst)
+            else:
+                shutil.copytree(self, dst)
+
+    def move(self, dst, if_exists=None):
+        return self.rename2(dst, if_exists)
+
+    def rename2(self, dst, if_exists=None):
         """ 相比原版的reanme，搞了更多骚操作，但性能也会略微下降，所以重写一个功能名 """
         if not self.exists():
             return
@@ -1153,19 +1234,7 @@ class XlPath(type(pathlib.Path())):
                 self.rename(tmp)
                 self.delete()
                 tmp.rename(dst)
-        elif dst.exists():
-            if if_exists is None:
-                self.rename(dst)
-            elif if_exists == 'error':
-                raise FileExistsError(f'{dst}')
-            elif if_exists == 'replace':
-                dst.delete()
-                self.rename(dst)
-            elif if_exists == 'skip':
-                pass
-            else:
-                raise ValueError(f'{if_exists}')
-        else:
+        elif dst.exist_preprcs(if_exists):
             self.rename(dst)
 
     def delete(self):
@@ -1174,34 +1243,77 @@ class XlPath(type(pathlib.Path())):
         elif self.is_dir():
             shutil.rmtree(self)
 
+    def backup(self, tail=None, if_exists='replace', move=False):
+        r""" 对文件末尾添加时间戳备份，也可以使用自定义标记tail
+
+        :param tail: 自定义添加后缀
+            tail为None时，默认添加特定格式的时间戳
+        :param if_exists: 备份的目标文件名存在时的处理方案
+            这个概率非常小，真遇到，先把已存在的删掉，重新写入一个是可以接受的
+        :param move: 是否删除原始文件
+
+        # TODO：有个小bug，如果在不同时间实际都是相同一个文件，也会被不断反复备份
+        #    如果想解决这个，就要读取目录下最近的备份文件对比内容了
+        """
+        from datetime import datetime
+
+        # 1 判断自身文件是否存在
+        if not self:
+            return None
+
+        # 2 计算出新名称
+        if not tail:
+            tail = datetime.fromtimestamp(self.mtime()).strftime(' %y%m%d-%H%M%S')  # 时间戳
+        name, ext = os.path.splitext(str(self))
+        dst = name + tail + ext
+
+        # 3 备份就是特殊的copy操作
+        if move:
+            return self.move(dst, if_exists)
+        else:
+            return self.copy(dst, if_exists)
+
     def __4_重复文件相关功能(self):
         """ 检查目录里的各种文件情况 """
 
-    def glob_repeat_files(self, pattern='*', *, sort_mode='count', print_mode=False):
+    def glob_repeat_files(self, pattern='*', *, sort_mode='count', print_mode=False,
+                          files=None, hash_func=None):
         """ 返回重复的文件组
 
+        :param files: 直接指定候选文件清单，此时pattern默认失效
+        :param hash_func: hash规则，默认使用etag规则
         :param sort_mode:
             count: 按照重复的文件数量从多到少排序
             size: 按照空间总占用量从大到小排序
         :return: [(etag, files, per_file_size), ...]
         """
+        # 0 文件清单和hash方法
+        if files is None:
+            files = list(self.glob_files(pattern))
+
+        if hash_func is None:
+            def hash_func(f):
+                return get_etag(str(f))
+
         # 1 获取所有etag，这一步比较费时
-        etag2files = defaultdict(list)
-        for f in tqdm(self.glob_files(pattern), desc='get etags', disable=not print_mode):
-            etag = get_etag(str(f))
-            etag2files[etag].append(f)
+        hash2files = defaultdict(list)
+
+        for f in tqdm(files, desc='get etags', disable=not print_mode):
+            etag = hash_func(f)
+            hash2files[etag].append(f)
 
         # 2 转格式，排序
-        etag2files = [(k, vs, vs[0].size()) for k, vs in etag2files.items() if len(vs) > 1]
+        hash2files = [(k, vs, vs[0].size()) for k, vs in hash2files.items() if len(vs) > 1]
         if sort_mode == 'count':
-            etag2files.sort(key=lambda x: (-len(x[1]), -len(x[1]) * x[2]))
+            hash2files.sort(key=lambda x: (-len(x[1]), -len(x[1]) * x[2]))
         elif sort_mode == 'size':
-            etag2files.sort(key=lambda x: (-len(x[1]) * x[2], -len(x[1])))
+            hash2files.sort(key=lambda x: (-len(x[1]) * x[2], -len(x[1])))
 
         # 3 返回每一组数据
-        return etag2files
+        return hash2files
 
-    def delete_repeat_files(self, pattern='*', *, sort_mode='count', print_mode=True, debug=False):
+    def delete_repeat_files(self, pattern='*', *, sort_mode='count', print_mode=True, debug=False,
+                            files=None, hash_func=None):
         """
         :param debug:
             True，只是输出检查清单，不做操作
@@ -1216,23 +1328,34 @@ class XlPath(type(pathlib.Path())):
             if print_mode:
                 print(*args, **kwargs)
 
-        files = self.glob_repeat_files(pattern, sort_mode=sort_mode, print_mode=print_mode)
+        files = self.glob_repeat_files(pattern, sort_mode=sort_mode, print_mode=print_mode,
+                                       files=files, hash_func=hash_func)
         for i, (etag, files, _size) in enumerate(files, start=1):
             n = len(files)
             printf(f'{i}、{etag}\t{format_size(_size)} × {n} ≈ {format_size(_size * n)}')
 
             for j, f in enumerate(files, start=1):
                 if debug:
-                    printf(f'\t{f}')
+                    printf(f'\t{f.relpath(self)}')
                 else:
                     if j == 1:
-                        printf(f'\t{f}')
+                        printf(f'\t{f.relpath(self)}')
                     else:
                         f.delete()
-                        printf(f'\t{f}\tdelete')
+                        printf(f'\t{f.relpath(self)}\tdelete')
             if print_mode:
                 printf()
 
+    def check_repeat_files(self, pattern='**/*', **kwargs):
+        if 'debug' not in kwargs:
+            kwargs['debug'] = True
+        self.delete_repeat_files(pattern, **kwargs)
+
+    def check_repeat_name_files(self, pattern='**/*', **kwargs):
+        if 'hash_func' not in kwargs:
+            kwargs['hash_func'] = lambda p: p.name.lower()
+        self.check_repeat_files(pattern, **kwargs)
+
     def __5_文件后缀相关功能(self):
         """ 检查目录里的各种文件情况 """
 
